#ifndef TP3LOG_H
#define TP3LOG_H

#include <QDebug>
#include <QObject>
#include <QIODevice>
#include <QFile>
#include <QVector>
#include <QRegExp>

#ifdef TP3LOG_DLL

#  if defined(TP3LOG_MAKEDLL)     // create a tp3Log DLL library
#    define TP3LOG_EXPORT Q_DECL_EXPORT
#  else                           // use a tp3Log DLL library
#    define TP3LOG_EXPORT Q_DECL_IMPORT
#  endif

#endif // TP3LOG_DLL

#ifndef TP3LOG_EXPORT
#  define TP3LOG_EXPORT
#endif

class tp3Debug;
class tp3DebugDevice;

class TP3LOG_EXPORT tp3Log : public QObject
{
    Q_OBJECT
public:
    enum LogLevel {
        LVL_FATAL = 0,
        LVL_WARNING,
        LVL_INFO,
        LVL_DEBUG,
        LVL_ALL
    };

    static tp3Log &instance() {
        static tp3Log log;
        return log;
    }

    static void setLogPath(QString path);
    static QString logPath();
    static QString oldLogPath();

    static QDebug debugStream(tp3Debug stream, QString tag, QString file, int line);

    static tp3Debug debug();
    static tp3Debug info();
    static tp3Debug warning();
    static tp3Debug fatal();

    static void include(QString regex) {
        instance().mIncludes.push_back(QRegExp(regex));
    }

    static void resetInclude() {
        instance().mIncludes.clear();
    }

    static void exclude(QString regex) {
        instance().mExcludes.push_back(QRegExp(regex));
    }

    static void resetExcludes() {
        instance().mExcludes.clear();
    }



    static LogLevel logLevel() {return instance().mLogLevel;}
    static void setLogLevel(const LogLevel level) {instance().mLogLevel = level;}


private:
    static bool ignore(tp3Debug &stream, QString tag);

    LogLevel mLogLevel = LVL_INFO;
    QVector<QRegExp> mIncludes;
    QVector<QRegExp> mExcludes;
};

class TP3LOG_EXPORT tp3Debug : public QDebug {
public:
    tp3Debug(tp3DebugDevice *device);
    ~tp3Debug();

    tp3Log::LogLevel level() const;
private:
    tp3DebugDevice *mDevice;
};

class TP3LOG_EXPORT tp3ZeroDebug {
public:
    template <typename T>
    inline tp3ZeroDebug &operator<<(T) {
        return *this;
    }
};


#define DEBUG tp3Log::debug()
#define INFO tp3Log::info()
#define WARNING tp3Log::warning()
#define FATAL tp3Log::fatal()

#define TP3_BUFFER_SIZE 1024

#define LOG(x) tp3Log::debugStream((x), QStringLiteral(__FILE__), QStringLiteral(__FILE__), __LINE__)
#define LOGT(x, tag) tp3Log::debugStream((x), tag, QStringLiteral(__FILE__), __LINE__)
#define LOGVA(x, ...)  {char b[TP3_BUFFER_SIZE]; snprintf(b, TP3_BUFFER_SIZE, __VA_ARGS__); LOG(x) << QLatin1Literal(b);}

#if defined(QT_DEBUG)
#define DLOG(x) LOG(x)
#define DLOGT(x, tag) LOG(x, tag)
#define DLOGVA(x, ...) LOGVA(x, ...)
#else
#define DLOG(x) tp3ZeroDebug()
#define DLOGT(x, tag) tp3ZeroDebug()
#define DLOGVA(x, ...)
#endif

#endif // TP3LOG_H
